@@ -1,19 +1,16 @@
-# chuk_llm/configuration/unified_config.py
 """
-Unified Configuration System for ChukLLM
-========================================
+Clean Unified Configuration System with Transparent Discovery
+===========================================================
 
 Single source of truth for all provider configuration, capabilities, and validation.
 Everything comes from one YAML file with intelligent merging and inheritance.
+Discovery is completely transparent - no API changes needed.
 """
 
 import os
-import re
 import logging
 from pathlib import Path
 from typing import Dict, List, Optional, Set, Any, Tuple, Union
-from dataclasses import dataclass, field
-from enum import Enum
 
 try:
     import yaml
@@ -26,123 +23,42 @@ try:
 except ImportError:
     _dotenv_available = False
 
+# Modern package resource handling
+try:
+    from importlib.resources import files
+    _importlib_resources_available = True
+except ImportError:
+    try:
+        from importlib_resources import files
+        _importlib_resources_available = True
+    except ImportError:
+        _importlib_resources_available = False
+
+# Fallback package resource handling
+try:
+    import pkg_resources
+    _pkg_resources_available = True
+except ImportError:
+    _pkg_resources_available = False
+
+from .models import Feature, ModelCapabilities, ProviderConfig
+from .discovery import ConfigDiscoveryMixin
+from .validator import ConfigValidator
+
 logger = logging.getLogger(__name__)
 
 
-# ──────────────────────────── Feature Definitions ─────────────────────────────
-class Feature(str, Enum):
-    """Supported LLM features"""
-    TEXT = "text"                          # Basic text completion capability
-    STREAMING = "streaming"                # Streaming response capability
-    TOOLS = "tools"                        # Function calling/tools
-    VISION = "vision"                      # Image/visual input processing
-    JSON_MODE = "json_mode"                # Structured JSON output
-    PARALLEL_CALLS = "parallel_calls"      # Multiple simultaneous function calls
-    SYSTEM_MESSAGES = "system_messages"    # System message support
-    MULTIMODAL = "multimodal"              # Multiple input modalities
-    REASONING = "reasoning"                # Advanced reasoning capabilities
-
-    @classmethod
-    def from_string(cls, value: str) -> "Feature":
-        """Convert string to Feature enum"""
-        try:
-            return cls(value.lower())
-        except ValueError as exc:
-            raise ValueError(f"Unknown feature: {value}") from exc
-
-
-# ──────────────────────────── Data Classes ─────────────────────────────
-@dataclass
-class ModelCapabilities:
-    """Model-specific capabilities with inheritance from provider"""
-    pattern: str
-    features: Set[Feature] = field(default_factory=set)
-    max_context_length: Optional[int] = None
-    max_output_tokens: Optional[int] = None
+class UnifiedConfigManager(ConfigDiscoveryMixin):
+    """
+    Unified configuration manager with transparent dynamic discovery.
     
-    def matches(self, model_name: str) -> bool:
-        """Check if this capability applies to the given model"""
-        return bool(re.match(self.pattern, model_name, flags=re.IGNORECASE))
-    
-    def get_effective_features(self, provider_features: Set[Feature]) -> Set[Feature]:
-        """Get effective features by inheriting from provider and adding model-specific"""
-        return provider_features.union(self.features)
-
-
-@dataclass 
-class ProviderConfig:
-    """Complete unified provider configuration"""
-    name: str
-    
-    # Client configuration
-    client_class: str = ""
-    api_key_env: Optional[str] = None
-    api_key_fallback_env: Optional[str] = None
-    api_base: Optional[str] = None
-    
-    # Model configuration
-    default_model: str = ""
-    models: List[str] = field(default_factory=list)
-    model_aliases: Dict[str, str] = field(default_factory=dict)
-    
-    # Provider-level capabilities (baseline for all models)
-    features: Set[Feature] = field(default_factory=set)
-    max_context_length: Optional[int] = None
-    max_output_tokens: Optional[int] = None
-    rate_limits: Dict[str, int] = field(default_factory=dict)
-    
-    # Model-specific capability overrides
-    model_capabilities: List[ModelCapabilities] = field(default_factory=list)
-    
-    # Inheritance and extras
-    inherits: Optional[str] = None
-    extra: Dict[str, Any] = field(default_factory=dict)
-    
-    def supports_feature(self, feature: Union[str, Feature], model: Optional[str] = None) -> bool:
-        """Check if provider/model supports a feature"""
-        if isinstance(feature, str):
-            feature = Feature.from_string(feature)
-        
-        if model:
-            # Check model-specific capabilities
-            model_caps = self.get_model_capabilities(model)
-            effective_features = model_caps.get_effective_features(self.features)
-            return feature in effective_features
-        else:
-            # Check provider baseline
-            return feature in self.features
-    
-    def get_model_capabilities(self, model: Optional[str] = None) -> ModelCapabilities:
-        """Get capabilities for specific model"""
-        if model and self.model_capabilities:
-            for mc in self.model_capabilities:
-                if mc.matches(model):
-                    # Return model-specific caps with proper inheritance
-                    return ModelCapabilities(
-                        pattern=mc.pattern,
-                        features=mc.get_effective_features(self.features),
-                        max_context_length=mc.max_context_length or self.max_context_length,
-                        max_output_tokens=mc.max_output_tokens or self.max_output_tokens
-                    )
-        
-        # Return provider defaults
-        return ModelCapabilities(
-            pattern=".*",
-            features=self.features.copy(),
-            max_context_length=self.max_context_length,
-            max_output_tokens=self.max_output_tokens
-        )
-    
-    def get_rate_limit(self, tier: str = "default") -> Optional[int]:
-        """Get rate limit for tier"""
-        return self.rate_limits.get(tier)
-
-
-# ──────────────────────────── Configuration Manager ─────────────────────────────
-class UnifiedConfigManager:
-    """Unified configuration manager with validation and capabilities"""
+    Discovery is completely seamless - existing APIs work unchanged.
+    All configuration comes from YAML, nothing hardcoded.
+    """
     
     def __init__(self, config_path: Optional[str] = None):
+        super().__init__()  # Initialize discovery mixin
+        
         self.config_path = config_path
         self.providers: Dict[str, ProviderConfig] = {}
         self.global_aliases: Dict[str, str] = {}
@@ -151,8 +67,6 @@ class UnifiedConfigManager:
         
         # Load environment variables first
         self._load_environment()
-        
-        # No built-in defaults - everything comes from config files
     
     def _load_environment(self):
         """Load environment variables from .env file"""
@@ -177,41 +91,220 @@ class UnifiedConfigManager:
         else:
             logger.debug("No .env file found in standard locations")
     
-    def _find_config_file(self) -> Optional[Path]:
-        """Find YAML configuration file in correct priority order"""
+    def _find_config_files(self) -> tuple[Optional[Path], Optional[Path]]:
+        """Find configuration files - returns (user_config, package_config)"""
+        user_config = None
+        package_config = None
+        
+        # 1. Look for user configuration files (these override/replace)
         if self.config_path:
             path = Path(self.config_path)
             if path.exists():
-                return path
+                user_config = path
         
-        candidates = [
-            # 1. Environment variable with location of chuk_llm.yaml
-            os.getenv("CHUK_LLM_CONFIG"),
+        if not user_config:
+            user_candidates = [
+                # Environment variable with location
+                os.getenv("CHUK_LLM_CONFIG"),
+                
+                # Working directory of consuming project
+                "chuk_llm.yaml",      # REPLACES package config completely
+                "providers.yaml",     # EXTENDS package config (inherits + overrides)
+                "llm_config.yaml",    # EXTENDS package config
+                "config/chuk_llm.yaml",
+                Path.home() / ".chuk_llm" / "config.yaml",
+            ]
             
-            # 2. Working directory of consuming project
-            "chuk_llm.yaml",
-            
-            # 3. ChukLLM hosted file in chuk_llm/chuk_llm.yaml
-            Path(__file__).parent.parent / "chuk_llm.yaml",
-            
-            # Additional fallbacks (keeping existing behavior)
-            "providers.yaml", 
-            "llm_config.yaml",
-            "config/chuk_llm.yaml",
-            Path(__file__).parent.parent / "providers.yaml",
-            Path.home() / ".chuk_llm" / "config.yaml",
-        ]
+            for candidate in user_candidates:
+                if candidate:
+                    path = Path(candidate).expanduser().resolve()
+                    if path.exists():
+                        logger.info(f"Found user config file: {path}")
+                        user_config = path
+                        break
         
-        for candidate in candidates:
-            if candidate:
-                path = Path(candidate).expanduser().resolve()
-                if path.exists():
-                    logger.info(f"Found config file: {path}")
-                    return path
+        # 2. Always try to get package config as baseline
+        package_config = self._get_package_config_path()
+        if package_config:
+            package_config = Path(package_config)
+            
+        return user_config, package_config
+
+    def _load_yaml_files(self) -> Dict:
+        """Load YAML configuration with inheritance logic"""
+        if not yaml:
+            logger.warning("PyYAML not available, using built-in defaults only")
+            return {}
+        
+        user_config_file, package_config_file = self._find_config_files()
+        
+        config = {}
+        
+        # 1. Start with package config as baseline (if available)
+        if package_config_file and package_config_file.exists():
+            try:
+                package_config = self._load_single_yaml(package_config_file)
+                if package_config:
+                    config.update(package_config)
+                    logger.info(f"Loaded package config from {package_config_file}")
+            except Exception as e:
+                logger.error(f"Failed to load package config from {package_config_file}: {e}")
+        
+        # 2. Handle user config based on filename
+        if user_config_file:
+            try:
+                user_config = self._load_single_yaml(user_config_file)
+                if user_config:
+                    filename = user_config_file.name
+                    
+                    if filename == "chuk_llm.yaml":
+                        # COMPLETE REPLACEMENT - ignore package config
+                        logger.info(f"chuk_llm.yaml found - replacing package config completely")
+                        config = user_config
+                    else:
+                        # INHERITANCE MODE - merge with package config
+                        logger.info(f"{filename} found - extending package config")
+                        config = self._merge_configs(config, user_config)
+                    
+                    logger.info(f"Loaded user config from {user_config_file}")
+            except Exception as e:
+                logger.error(f"Failed to load user config from {user_config_file}: {e}")
+        
+        if not config:
+            logger.info("No configuration files found, using built-in defaults")
+        
+        return config
+
+    def _load_single_yaml(self, config_file: Path) -> Dict:
+        """Load a single YAML file"""
+        # Handle package resource files specially
+        if self._is_package_resource_path(config_file):
+            return self._load_package_yaml()
+        
+        # Regular file loading
+        with open(config_file, 'r', encoding='utf-8') as f:
+            return yaml.safe_load(f) or {}
+
+    def _merge_configs(self, base_config: Dict, user_config: Dict) -> Dict:
+        """Merge user config into base config with intelligent merging"""
+        merged = base_config.copy()
+        
+        for key, value in user_config.items():
+            if key.startswith("__"):
+                # Global sections - merge intelligently
+                if key in merged and isinstance(merged[key], dict) and isinstance(value, dict):
+                    # Merge global sections (like __global__, __global_aliases__)
+                    merged[key].update(value)
+                else:
+                    # Replace if not both dicts
+                    merged[key] = value
+            else:
+                # Provider sections
+                if key in merged and isinstance(merged[key], dict) and isinstance(value, dict):
+                    # Merge provider config - user overrides base
+                    provider_config = merged[key].copy()
+                    
+                    # Handle special cases for lists that should be extended vs replaced
+                    for sub_key, sub_value in value.items():
+                        if sub_key == "models" and isinstance(sub_value, list) and isinstance(provider_config.get(sub_key), list):
+                            # EXTEND models list (unique values only)
+                            existing_models = set(provider_config[sub_key])
+                            new_models = [m for m in sub_value if m not in existing_models]
+                            provider_config[sub_key].extend(new_models)
+                        elif sub_key == "model_aliases" and isinstance(sub_value, dict) and isinstance(provider_config.get(sub_key), dict):
+                            # MERGE model aliases
+                            provider_config[sub_key].update(sub_value)
+                        elif sub_key == "features" and isinstance(sub_value, list) and isinstance(provider_config.get(sub_key), list):
+                            # EXTEND features list (unique values only)
+                            existing_features = set(provider_config[sub_key])
+                            new_features = [f for f in sub_value if f not in existing_features]
+                            provider_config[sub_key].extend(new_features)
+                        elif sub_key == "rate_limits" and isinstance(sub_value, dict) and isinstance(provider_config.get(sub_key), dict):
+                            # MERGE rate limits
+                            provider_config[sub_key].update(sub_value)
+                        elif sub_key == "extra" and isinstance(sub_value, dict) and isinstance(provider_config.get(sub_key), dict):
+                            # DEEP MERGE extra config
+                            provider_config[sub_key] = self._deep_merge_dict(provider_config[sub_key], sub_value)
+                        else:
+                            # REPLACE for other fields
+                            provider_config[sub_key] = sub_value
+                    
+                    merged[key] = provider_config
+                else:
+                    # Replace entire provider config if not both dicts
+                    merged[key] = value
+        
+        return merged
+
+    def _deep_merge_dict(self, base: Dict, override: Dict) -> Dict:
+        """Deep merge two dictionaries"""
+        result = base.copy()
+        
+        for key, value in override.items():
+            if key in result and isinstance(result[key], dict) and isinstance(value, dict):
+                result[key] = self._deep_merge_dict(result[key], value)
+            else:
+                result[key] = value
+        
+        return result
+
+    def _get_package_config_path(self) -> Optional[str]:
+        """Get the path to the packaged chuk_llm.yaml file"""
+        # Try modern importlib.resources first
+        if _importlib_resources_available:
+            try:
+                package_files = files('chuk_llm')
+                config_file = package_files / 'chuk_llm.yaml'
+                if config_file.is_file():
+                    # Return a temporary path for importlib.resources
+                    return str(config_file)
+            except Exception as e:
+                logger.debug(f"importlib.resources method failed: {e}")
+        
+        # Fallback to pkg_resources
+        if _pkg_resources_available:
+            try:
+                if pkg_resources.resource_exists('chuk_llm', 'chuk_llm.yaml'):
+                    return pkg_resources.resource_filename('chuk_llm', 'chuk_llm.yaml')
+            except Exception as e:
+                logger.debug(f"pkg_resources method failed: {e}")
         
-        logger.warning("No configuration file found in any standard location")
         return None
 
+    def _is_package_resource_path(self, config_file: Path) -> bool:
+        """Check if this is a package resource path that needs special handling"""
+        config_str = str(config_file)
+        # Check for importlib.resources paths (they often contain special markers)
+        return (not config_file.exists() and 
+                ('chuk_llm' in config_str or 'importlib' in config_str))
+
+    def _load_package_yaml(self) -> Dict:
+        """Load YAML from package resources"""
+        # Try modern importlib.resources first
+        if _importlib_resources_available:
+            try:
+                package_files = files('chuk_llm')
+                config_file = package_files / 'chuk_llm.yaml'
+                if config_file.is_file():
+                    content = config_file.read_text(encoding='utf-8')
+                    config = yaml.safe_load(content) or {}
+                    logger.info("Loaded configuration from package resources (importlib)")
+                    return config
+            except Exception as e:
+                logger.debug(f"importlib.resources loading failed: {e}")
+        
+        # Fallback to pkg_resources
+        if _pkg_resources_available:
+            try:
+                content = pkg_resources.resource_string('chuk_llm', 'chuk_llm.yaml').decode('utf-8')
+                config = yaml.safe_load(content) or {}
+                logger.info("Loaded configuration from package resources (pkg_resources)")
+                return config
+            except Exception as e:
+                logger.debug(f"pkg_resources loading failed: {e}")
+        
+        return {}
+
     def _parse_features(self, features_data: Any) -> Set[Feature]:
         """Parse features from YAML data"""
         if not features_data:
@@ -247,25 +340,8 @@ class UnifiedConfigManager:
         return capabilities
     
     def _load_yaml(self) -> Dict:
-        """Load YAML configuration"""
-        if not yaml:
-            logger.warning("PyYAML not available, using built-in defaults only")
-            return {}
-        
-        config_file = self._find_config_file()
-        if not config_file:
-            logger.info("No configuration file found, using built-in defaults")
-            return {}
-        
-        try:
-            with open(config_file, 'r', encoding='utf-8') as f:
-                config = yaml.safe_load(f) or {}
-            
-            logger.info(f"Loaded configuration from {config_file}")
-            return config
-        except Exception as exc:
-            logger.error(f"Failed to load config from {config_file}: {exc}")
-            return {}
+        """Load YAML configuration with inheritance support"""
+        return self._load_yaml_files()
     
     def _process_config(self, config: Dict):
         """Process YAML configuration and merge with defaults"""
@@ -320,13 +396,37 @@ class UnifiedConfigManager:
             if "inherits" in data:
                 provider.inherits = data["inherits"]
             
-            # Extra fields
-            extra_fields = {k: v for k, v in data.items() 
-                          if k not in {"client_class", "api_key_env", "api_key_fallback_env", 
-                                      "api_base", "default_model", "models", "model_aliases",
-                                      "features", "max_context_length", "max_output_tokens", 
-                                      "rate_limits", "model_capabilities", "inherits"}}
-            provider.extra.update(extra_fields)
+            # CRITICAL FIX: Process extra fields INSIDE the provider loop
+            known_fields = {"client_class", "api_key_env", "api_key_fallback_env",
+                           "api_base", "default_model", "models", "model_aliases",
+                           "features", "max_context_length", "max_output_tokens",
+                           "rate_limits", "model_capabilities", "inherits"}
+
+            # Extract extra fields for THIS provider
+            extra_fields = {k: v for k, v in data.items() if k not in known_fields}
+
+            # CRITICAL FIX: Deep merge extra fields for THIS provider
+            for key, value in extra_fields.items():
+                if isinstance(value, dict) and key in provider.extra and isinstance(provider.extra[key], dict):
+                    # Deep merge dictionaries (like dynamic_discovery)
+                    provider.extra[key].update(value)
+                else:
+                    # Replace for non-dict values
+                    provider.extra[key] = value
+            
+            # CRITICAL FIX: Handle double nesting issue
+            if "extra" in provider.extra and isinstance(provider.extra["extra"], dict):
+                # Flatten double-nested extra fields
+                nested_extra = provider.extra["extra"]
+                del provider.extra["extra"]
+                provider.extra.update(nested_extra)
+                logger.debug(f"Fixed double nesting for provider {name}")
+            
+            # Debug logging for discovery config
+            if "dynamic_discovery" in extra_fields:
+                logger.debug(f"Added discovery config to {name}: {extra_fields['dynamic_discovery']}")
+            elif "dynamic_discovery" in provider.extra:
+                logger.debug(f"Discovery config available for {name}: enabled={provider.extra['dynamic_discovery'].get('enabled')}")
     
     def _resolve_inheritance(self):
         """Resolve provider inheritance - inherit config but NOT models/aliases"""
@@ -375,7 +475,7 @@ class UnifiedConfigManager:
             
             if not changes:
                 break
-            
+    
     def load(self):
         """Load configuration"""
         if self._loaded:
@@ -387,11 +487,13 @@ class UnifiedConfigManager:
         self._loaded = True
     
     def get_provider(self, name: str) -> ProviderConfig:
-        """Get provider configuration"""
+        """Get provider configuration (with transparent discovery)"""
         self.load()
+        
         if name not in self.providers:
             available = ", ".join(self.providers.keys())
             raise ValueError(f"Unknown provider: {name}. Available: {available}")
+        
         return self.providers[name]
     
     def get_all_providers(self) -> List[str]:
@@ -445,111 +547,10 @@ class UnifiedConfigManager:
         self.providers.clear()
         self.global_aliases.clear()
         self.global_settings.clear()
+        super().reload()  # Clear discovery state
         self.load()
 
 
-# ──────────────────────────── Validation ─────────────────────────────
-class ConfigValidator:
-    """Validates configurations and requests"""
-    
-    @staticmethod
-    def validate_provider_config(provider: ProviderConfig, strict: bool = False) -> Tuple[bool, List[str]]:
-        """Validate provider configuration"""
-        issues = []
-        
-        # Check required fields
-        if not provider.client_class:
-            issues.append(f"Missing 'client_class' for provider {provider.name}")
-        
-        # Check API key for non-local providers
-        if provider.name not in ["ollama", "local"]:
-            if provider.api_key_env and not os.getenv(provider.api_key_env):
-                if not provider.api_key_fallback_env or not os.getenv(provider.api_key_fallback_env):
-                    issues.append(f"Missing API key: {provider.api_key_env} environment variable not set")
-        
-        # Validate API base URL
-        if provider.api_base and not ConfigValidator._is_valid_url(provider.api_base):
-            issues.append(f"Invalid API base URL: {provider.api_base}")
-        
-        # Check default model
-        if not provider.default_model:
-            issues.append(f"Missing 'default_model' for provider {provider.name}")
-        
-        return len(issues) == 0, issues
-    
-    @staticmethod
-    def validate_request_compatibility(
-        provider_name: str,
-        model: Optional[str] = None,
-        messages: Optional[List[Dict[str, Any]]] = None,
-        tools: Optional[List[Dict[str, Any]]] = None,
-        stream: bool = False,
-        **kwargs
-    ) -> Tuple[bool, List[str]]:
-        """Validate if request is compatible with provider/model"""
-        issues = []
-        
-        try:
-            config_manager = get_config()
-            provider = config_manager.get_provider(provider_name)
-            
-            # Check streaming support
-            if stream and not provider.supports_feature(Feature.STREAMING, model):
-                issues.append(f"{provider_name}/{model or 'default'} doesn't support streaming")
-            
-            # Check tools support
-            if tools and not provider.supports_feature(Feature.TOOLS, model):
-                issues.append(f"{provider_name}/{model or 'default'} doesn't support function calling")
-            
-            # Check vision support
-            if messages and ConfigValidator._has_vision_content(messages):
-                if not provider.supports_feature(Feature.VISION, model):
-                    issues.append(f"{provider_name}/{model or 'default'} doesn't support vision/image inputs")
-            
-            # Check JSON mode
-            if kwargs.get("response_format") == "json":
-                if not provider.supports_feature(Feature.JSON_MODE, model):
-                    issues.append(f"{provider_name}/{model or 'default'} doesn't support JSON mode")
-            
-        except Exception as exc:
-            issues.append(f"Configuration error: {exc}")
-        
-        return len(issues) == 0, issues
-    
-    @staticmethod
-    def _is_valid_url(url: str) -> bool:
-        """Basic URL validation"""
-        if not url:
-            return False
-        
-        url_pattern = re.compile(
-            r'^https?://'
-            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'
-            r'localhost|'
-            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
-            r'(?::\d+)?'
-            r'(?:/?|[/?]\S+)$',
-            re.IGNORECASE
-        )
-        return url_pattern.match(url) is not None
-    
-    @staticmethod
-    def _has_vision_content(messages: List[Dict[str, Any]]) -> bool:
-        """Check if messages contain vision/image content"""
-        if not messages:
-            return False
-        
-        for message in messages:
-            if not message:
-                continue
-            content = message.get("content", "")
-            if isinstance(content, list):
-                for item in content:
-                    if isinstance(item, dict) and item.get("type") in ["image", "image_url"]:
-                        return True
-        return False
-
-
 # ──────────────────────────── Capability Checker ─────────────────────────────
 class CapabilityChecker:
     """Query helpers for provider capabilities"""
